@@ -4,7 +4,7 @@ Provides the FileList class, used for poking about the filesystem
 and building lists of files.
 """
 
-__revision__ = "$Id: filelist.py 65250 2008-07-26 20:09:45Z amaury.forgeotdarc $"
+__revision__ = "$Id: filelist.py 76045 2009-11-01 23:11:55Z tarek.ziade $"
 
 import os, re
 import fnmatch
@@ -289,7 +289,8 @@ def glob_to_re(pattern):
     # character except the special characters.
     # XXX currently the "special characters" are just slash -- i.e. this is
     # Unix-only.
-    pattern_re = re.sub(r'(^|[^\\])\.', r'\1[^/]', pattern_re)
+    pattern_re = re.sub(r'((?<!\\)(\\\\)*)\.', r'\1[^/]', pattern_re)
+
     return pattern_re
 
 
@@ -311,7 +312,9 @@ def translate_pattern(pattern, anchor=1, prefix=None, is_regex=0):
         pattern_re = ''
 
     if prefix is not None:
-        prefix_re = (glob_to_re(prefix))[0:-1] # ditch trailing $
+        # ditch end of pattern character
+        empty_pattern = glob_to_re('')
+        prefix_re = (glob_to_re(prefix))[:-len(empty_pattern)]
         pattern_re = "^" + os.path.join(prefix_re, ".*" + pattern_re)
     else:                               # no prefix -- respect anchor flag
         if anchor:
