@@ -8,7 +8,7 @@ import warnings
 class TestUntestedModules(unittest.TestCase):
     def test_at_least_import_untested_modules(self):
         with warnings.catch_warnings():
-            import aifc
+            warnings.simplefilter("ignore")
             import bdb
             import cgitb
             import code
@@ -21,7 +21,6 @@ class TestUntestedModules(unittest.TestCase):
             import distutils.filelist
             if sys.platform.startswith('win'):
                 import distutils.msvccompiler
-            import distutils.mwerkscompiler
             import distutils.text_file
             import distutils.unixccompiler
 
@@ -60,10 +59,8 @@ class TestUntestedModules(unittest.TestCase):
             import pdb
             import pstats
             import py_compile
-            import rlcompleter
             import sched
             import sndhdr
-            import sunau
             import symbol
             import tabnanny
             import timeit
